@@ -5,12 +5,12 @@ import PackageDescription
 
 let package = Package(
   name: "nanopb",
-    platforms: [
-        .macOS(.v10_10),
-        .iOS(.v8),
-        .tvOS(.v9),
-        .watchOS(.v2)
-    ],
+  products: [
+    .library(
+      name: "nanopb",
+      targets: ["nanopb"]
+    )
+  ],
 
   targets: [
     .target(
@@ -25,7 +25,7 @@ let package = Package(
         "pb_encode.h",
         "pb_encode.c"
       ],
-      publicHeadersPath: "SwiftPackage",
+      publicHeadersPath: "spm_headers",
       cSettings: [
         .define("PB_FIELD_32BIT", to: "1"),
         .define("PB_NO_PACKED_STRUCTS", to: "1"),
