@@ -21,12 +21,13 @@ git archive HEAD | tar x -C $DEST
 ( cd $DEST/generator; python3 nanopb_generator.py ||: )
 
 # Package the Python libraries
-( cd $DEST/generator; python3 -m PyInstaller nanopb_generator.py )
-( cd $DEST/generator; python3 -m PyInstaller protoc  )
+( cd $DEST/generator; python3 -m PyInstaller --strip nanopb_generator.py )
+( cd $DEST/generator; python3 -m PyInstaller --strip protoc  )
 mv $DEST/generator/dist/nanopb_generator $DEST/generator-bin
 cp $DEST/generator/dist/protoc/protoc $DEST/generator-bin
 
 # Include Google's descriptor.proto and nanopb.proto
+mkdir -p $DEST/generator-bin/grpc_tools/
 cp -pr $(python3 -c 'import grpc_tools, os.path; print(os.path.dirname(grpc_tools.__file__))')/_proto $DEST/generator-bin/grpc_tools/
 cp -pr $DEST/generator/proto $DEST/generator-bin/proto
 
@@ -36,9 +37,6 @@ rm -rf $DEST/generator/dist $DEST/generator/build $DEST/generator/nanopb_generat
 # Make the nanopb generator available as a protoc plugin
 cp $DEST/generator-bin/nanopb_generator $DEST/generator-bin/protoc-gen-nanopb
 
-# Remove debugging symbols to reduce size of package
-( cd $DEST/generator-bin; strip *.so* )
-
 # Tar it all up
 ( cd dist; tar -czf $VERSION.tar.gz $VERSION )
 
